@@ -14,8 +14,7 @@ module Data.FileEmbed
     ) where
 
 import Language.Haskell.TH.Syntax
-    ( runQ
-    , Exp (AppE, ListE, LitE, TupE)
+    ( Exp (AppE, ListE, LitE, TupE)
 #if MIN_VERSION_template_haskell(2,5,0)
     , Lit (StringL, StringPrimL, IntegerL)
 #else
@@ -23,6 +22,9 @@ import Language.Haskell.TH.Syntax
 #endif
     , Q
     , runIO
+#if MIN_VERSION_template_haskell(2,7,0)
+    , Quasi(qAddDependentFile)
+#endif
     )
 import System.Directory (doesDirectoryExist, doesFileExist,
                          getDirectoryContents)
@@ -42,7 +44,11 @@ import System.IO.Unsafe (unsafePerformIO)
 -- > myFile :: Data.ByteString.ByteString
 -- > myFile = $(embedFile "dirName/fileName")
 embedFile :: FilePath -> Q Exp
-embedFile fp = (runIO $ B.readFile fp) >>= bsToExp
+embedFile fp =
+#if MIN_VERSION_template_haskell(2,7,0)
+    qAddDependentFile fp >>
+#endif
+  (runIO $ B.readFile fp) >>= bsToExp
 
 -- | Embed a directory recusrively in your source code.
 --
@@ -51,7 +57,7 @@ embedFile fp = (runIO $ B.readFile fp) >>= bsToExp
 -- > myDir :: [(FilePath, Data.ByteString.ByteString)]
 -- > myDir = $(embedDir "dirName")
 embedDir :: FilePath -> Q Exp
-embedDir fp = ListE <$> ((runIO $ fileList fp) >>= mapM pairToExp)
+embedDir fp = ListE <$> ((runIO $ fileList fp) >>= mapM (pairToExp fp))
 
 -- | Get a directory tree in the IO monad.
 --
@@ -59,8 +65,11 @@ embedDir fp = ListE <$> ((runIO $ fileList fp) >>= mapM pairToExp)
 getDir :: FilePath -> IO [(FilePath, B.ByteString)]
 getDir = fileList
 
-pairToExp :: (FilePath, B.ByteString) -> Q Exp
-pairToExp (path, bs) = do
+pairToExp :: FilePath -> (FilePath, B.ByteString) -> Q Exp
+pairToExp _root (path, bs) = do
+#if MIN_VERSION_template_haskell(2,7,0)
+    qAddDependentFile $ _root ++ '/' : path
+#endif
     exp' <- bsToExp bs
     return $! TupE [LitE $ StringL path, exp']
 
@@ -95,7 +104,10 @@ fileList' realTop top = do
 liftPair2 :: Monad m => (a, m b) -> m (a, b)
 liftPair2 (a, b) = b >>= \b' -> return (a, b')
 
+magic :: [Char]
 magic = concat ["fe", "MS"]
+
+sizeLen :: Int
 sizeLen = 20
 
 getInner :: B.ByteString -> B.ByteString
@@ -137,7 +149,7 @@ inject toInj orig =
     size = case reads $ B8.unpack sizeBS of
             (i, _):_ -> i
             [] -> error $ "Data.FileEmbed (inject): Your dummy space has been corrupted. Size is: " ++ show sizeBS
-    (dummy, after) = B.splitAt size rest'
+    after = B.drop size rest'
 
 injectFile :: B.ByteString
            -> FilePath -- ^ template file
